import os
import sys

import argparse

from hexdumper.Hexdump import Hexdump
from hexdumper.Controller import Controller

def main(args=None):
    """The main routine."""
    if args is None:
        args = sys.argv[1:]

    parser = argparse.ArgumentParser()
    parser.add_argument("--path", "-p", type=str, required=True, help="Path to file")
    parser.add_argument("--noHash", "-nH", action="store_false", help="Disable hashing e.g. for big files")
    args = parser.parse_args()

    c = Controller(args.path, args.noHash)

    c.printHeader()

    if(os.path.isfile(args.path)):
        hD = Hexdump(args.path)
        hD.print()
    else:
        print("The provided path is not a file!")
        pass      

    print("")
    print("######################################################################################################################")
    
    c.printExecutionTime()


if __name__ == "__main__":
    sys.exit(main())





class Hexdump():
    def __init__(self, path="") -> None:
        self.path = path
    
    def print(self):
        with open(self.path, "rb") as f:
            offset = 0
            self.printHexTableHeader()
            while True:
                bytes_ = f.read(16)
                if not bytes_:
                    break
                hexList = self.getHexList(bytes_, self.formatOffset(offset))
                self.printHexTableRow(hexList)
                offset += 16

    def printNoOfBytes(self, hexList):
        offset = 0
        # hexList contains hex values ["00", "0A", "FF", ...] etc.
        # We are going to split it into rows of 16 values for correct printing
        splitted = [hexList[x:x+16] for x in range(0, len(hexList), 16)]
        self.printHexTableHeader()
        
        for line in splitted:
            line.insert(0, self.formatOffset(offset))
            self.printHexTableRow(line)
            offset += 16

    def getHexList(self, bytes_, offset):
        # Convert the bytes to a list of hex values
        hexlist = [hex(x).split('x')[-1].upper() for x in list(bytes_)]
        result = [offset]

        for hex_ in hexlist:
            # While converting the bytes, the leading zeros gets cut of
            # the following instructions check if there are 2 characters
            # and adds a leading zero if necessary
            if (len(hex_) == 2):
                result.append(hex_)
            elif (len(hex_) == 1):
                result.append("0" + hex_)

        return result
            
    def printHexTableHeader(self):
        h = "{:10}     {:4} {:4} {:4} {:4} {:4} {:4} {:4} {:4}   {:4} {:4} {:4} {:4} {:4} {:4} {:4} {:4}      {:5}".format("    Offset", "00", "01", "02", "03", "04", "05",
                                                                                                                          "06", "07", "08", "09", "0A", "0B",
                                                                                                                          "0C", "0D", "0E", "0F", "ASCII")
        print(h)
        print("----------     -------------------------------------------------------------------------------        ----------------")

    def printHexTableRow(self, h):
        formatStr = "{:4}     {:4} {:4} {:4} {:4} {:4} {:4} {:4} {:4}   {:4} {:4} {:4} {:4} {:4} {:4} {:4} {:4}      {:1}{:1}{:1}{:1}{:1}{:1}{:1}{:1}{:1}{:1}{:1}{:1}{:1}{:1}{:1}{:1}"

        # Padding for lines with less than 16 bytes
        while True:
            if (len(h) < 17):
                h.append("  ")
            else:
                break

        a = [""]
        for i in range(1, 17):
            a.append(self.hexToAscii(h[i]))

        r = formatStr.format(h[0],
                             h[1], h[2],  h[3],  h[4],  h[5],  h[6],  h[7],  h[8],
                             h[9], h[10], h[11], h[12], h[13], h[14], h[15], h[16],
                             a[1], a[2],  a[3],  a[4],  a[5],  a[6],  a[7],  a[8],
                             a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16])
        print(r)

    def formatOffset(self, offset):
        r = hex(offset).split('x')[-1]
        return r.rjust(10, "0").upper()

    def hexToAscii(self, hex_):
        result = "."
        if (hex_ != "  "):
            d = int(hex_, 16)
            if (d >= 32 and d <= 126):
                result = chr(d)
        return result


from setuptools import setup

with open("README.md", "r") as r:
    desc = r.read()

setup(
    name="hexdumper",            
    version="1.0.1",
    author="5f0",
    url="https://github.com/5f0ne/hexdump",
    description="Prints the hexdump of a selected file",
    classifiers=[
        "Operating System :: OS Independent ",
        "Programming Language :: Python :: 3 ",
        "License :: OSI Approved :: MIT License "
    ],
    license="MIT",
    long_description=desc,
    long_description_content_type="text/markdown",
    package_dir={"": "src"},
    packages=["hexdumper"],
    install_requires=[
        "hash_calc"
    ],
    entry_points={
        "console_scripts": [
            "hexdumper = hexdumper.__main__:main"
        ]
    }
)


import os
import time

from datetime import datetime

from hash_calc.HashCalc import HashCalc

class Controller():
    def __init__(self, path, hashingAllowed) -> None:
        self.startTime = time.time()
        self.md5 = "-"
        self.sha256 = "-"
        self.path = path
        if(hashingAllowed):
            h = HashCalc(path)
            self.md5 = h.md5
            self.sha256 = h.sha256

    def printHeader(self):
        print("######################################################################################################################")
        print("")
        print("Hexdump by 5f0")
        print("Prints the hexdump of a selected file")
        print("")
        print("Current working directory: " + os.getcwd())
        print("        Investigated file: " + self.path)
        print("")
        print("                 MD5 Hash: " + self.md5)
        print("              SHA256 Hash: " + self.sha256)
        print("")
        print("                 Datetime: " + datetime.now().strftime("%d/%m/%Y %H:%M:%S"))
        print("")
        print("######################################################################################################################")
        print("")

    def printExecutionTime(self):
        end = time.time()
        print("")
        print("Execution Time: " + str(end-self.startTime)[0:8] + " sec")
        print("")